@@ -1,7 +1,7 @@
-require(RCurl)
+require(httr)
 #' API layer for the ODAM web services
 #'
-#' @author Daniel Jacob - INRA UMR 1332 BFP (C) 2019
+#' @author Daniel Jacob - INRAE UMR 1332 BFP (C) 2021
 #'
 #' @description the class that implements the API layer for the ODAM (Open Data for Access and Mining) web services.
 #'
@@ -10,12 +10,13 @@ require(RCurl)
 #' @rdname odamws
 #' @field wsURL defines the URL of the webservice - Must be specify when creating a new instance of the odamws object.
 #' @field dsname specifies the name of the Dataset to query - Must be specify when creating a new instance of the odamws object.
-#' @field delimiter specifies the delimiter used within data subset files 
 #' @field auth specifies the authentication code to access to the Dataset by this webservice (if required)
 #' @field subsets a data.frame object containing metadata related to the data subsets - Initialized during the instantiation step
 #' @field subsetNames a list of the data subset names - Initialized during the instantiation step
 #' @field connectList a matrix of the connection graph between data subsets (i.e. the links between each subset with the subset at its origin, so that links can be interpreted as 'obtained from'). The data subsets are referred by their subset number. (corresponding to the 'SetID' column in the 'subsets' field)  - Initialized during the instantiation step.
 #' @field msgError contains an error message if an error occurs
+#' @field maxtime defines the maximum request time
+#' @field ssl_verifypeer defines if the peer's SSL certificate is verified
 #' @examples
 #'\dontrun{
 #' dh <- new("odamws", "https://pmb-bordeaux.fr/getdata/", "frim1")
@@ -23,12 +24,12 @@ require(RCurl)
 #' # Get data from 'samples' subset with a constraint
 #' data <- dh$getDataByName('samples','sample/365')
 #' # Get 'activome' data subset
-#' ds <- dh$getSubsetByName('activome')
+#' ds <- dh$getSubsetByName('activome') 
 #' # Get the merged data of both data subsets based on their common identifiers
 #' setNameList <- c("activome", "qNMR_metabo" )
 #' dsMerged <- dh$getSubsetByName(setNameList)
 #'}
-#' @import RCurl
+#' @import httr
 #' @importFrom methods setRefClass
 #' @importFrom methods new
 #' @export
@@ -36,25 +37,27 @@ odamws <- setRefClass("odamws",
    fields = list(
       wsURL       = "character",
       dsname      = "character",
-      delimiter   = "character",
       auth        = "character",
       subsets     = "data.frame",
       subsetNames = "character",
       connectList = "matrix",
-      msgError    = "character"
+      msgError    = "character",
+      maxtime     = "numeric",
+      ssl_verifypeer = "logical"
    ), 
    methods = list(
    
       # Initialize the attributes
-      initialize = function(wsURL, dsname, auth='', delimiter="\t")
+      initialize = function(wsURL, dsname, auth='', maxtime=5, ssl_verifypeer = TRUE)
       {
          options(stringsAsFactors=FALSE)
          options(warn=-1)
          wsURL <<- wsURL
          dsname <<- dsname
          auth <<- auth
-         delimiter <<- delimiter
          msgError <<- ''
+         maxtime <<- maxtime
+         ssl_verifypeer <<- ssl_verifypeer
 
          # Get subsets information
          subsets <<- getWS('subset')
@@ -121,12 +124,25 @@ odamws <- setRefClass("odamws",
       {
       "Low level routine allowing to retrieve data or metadata from  a query formatted according the ODAM framework specifications - Returns a data.frame object. By default, i.e. with an empty query, a data.frame object containing metadata related to the data subsets is returned."
 
-         myurl <- paste(wsURL,'/tsv/', dsname, '/', query,"?auth=",auth,sep="");
-         out <- read.csv(textConnection(RCurl::getURL(myurl, ssl.verifypeer = FALSE)), head=TRUE, sep=delimiter);
-         if (dim(out)[1]==0) {
-             msgError <<- gsub("\\.", " ", colnames(out))[1]
+         myurl <- paste(wsURL, '/tsv/', dsname, '/', query, sep="")
+         headers <-  c();
+         if ( nchar(auth)>0 ) { headers <- c('x-api-key' = auth) }
+         out <- data.frame()
+         tryCatch({
+             resp <- httr::GET(myurl, config = httr::config(ssl_verifypeer = ssl_verifypeer),
+                                      add_headers(.headers = headers), timeout(maxtime))
+             T <- simplify2array(strsplit(httr::content(resp, as='text'),"\n"))
+             if (length(grep("(DOCTYPE|html)", T[1]))) {
+                 msgError <<- "ERROR: the query-path is not valid"
+             } else {
+                 out <- read.csv(textConnection(T), head=TRUE, sep="\t")
+                 if (dim(out)[1]==0) { msgError <<- gsub("\\.", " ", colnames(out))[1] }
+             }
+             if(nchar(msgError)>0) { cat(msgError) }
+         }, error=function(e) {
+             msgError <<- "ERROR : the API host is not responding; it is either not found or does not exist"
              cat(msgError)
-         }
+         })
          out
       },
 
@@ -174,7 +190,7 @@ odamws <- setRefClass("odamws",
          # Get DATA
          slash <- ifelse ( nchar(condition)==0 || substr(condition,1,1)=='/', '', '/' )
          data <- getWS(paste('(',strNameList,')',slash, condition,sep=''))
-         
+
          # Get quantitative variable features
          varnames <- NULL
          Q <- getWS(paste('(',strNameList,')/quantitative',sep=''))
@@ -221,16 +237,16 @@ odamws <- setRefClass("odamws",
          
          # Merge all labels
          LABELS <- rbind( 
-            matrix( c( as.matrix(samplename)[,c(1:3)], 'Identifier', as.matrix(samplename)[,c(5:6)]), ncol=6, byrow=FALSE  ),
-            matrix( c( as.matrix(facnames)[,c(1:3)], replicate(dim(facnames)[1],'Factor'  ), as.matrix(facnames)[,c(5:6)] ), ncol=6, byrow=FALSE  ),
-            matrix( c( as.matrix(varnames)[,c(1:3)], replicate(dim(varnames)[1],'Variable'), as.matrix(varnames)[,c(5:6)] ), ncol=6, byrow=FALSE  )
+            matrix( c( as.matrix(samplename)[,c(1:4)], 'Identifier', as.matrix(samplename)[,c(6:7)]), ncol=7, byrow=FALSE  ),
+            matrix( c( as.matrix(facnames)[,c(1:4)], replicate(dim(facnames)[1],'Factor'  ), as.matrix(facnames)[,c(6:7)] ), ncol=7, byrow=FALSE  ),
+            matrix( c( as.matrix(varnames)[,c(1:4)], replicate(dim(varnames)[1],'Variable'), as.matrix(varnames)[,c(6:7)] ), ncol=7, byrow=FALSE  )
          )
          if (dim(as.matrix(qualnames))[1]>0 ) { LABELS <- rbind ( LABELS, 
-            matrix( c( as.matrix(qualnames)[,c(1:3)], replicate(dim(qualnames)[1],'Feature'), as.matrix(qualnames)[,c(5:6)] ), ncol=6, byrow=FALSE )
+            matrix( c( as.matrix(qualnames)[,c(1:4)], replicate(dim(qualnames)[1],'Feature'), as.matrix(qualnames)[,c(6:7)] ), ncol=7, byrow=FALSE )
          )}
-         colnames(LABELS) <- c( 'Subset', 'Attribute', 'Description', 'Type', 'CV_Term_ID ', 'CV_Term_Name' )
-         LABELS[,5] <- sapply(CHAR(LABELS[,5]), function(x) { ifelse( ! is.na(x), x, "NA" ); })
+         colnames(LABELS) <- c( 'Subset', 'Attribute', 'WSEntry', 'Description', 'Type', 'CV_Term_ID ', 'CV_Term_Name' )
          LABELS[,6] <- sapply(CHAR(LABELS[,6]), function(x) { ifelse( ! is.na(x), x, "NA" ); })
+         LABELS[,7] <- sapply(CHAR(LABELS[,7]), function(x) { ifelse( ! is.na(x), x, "NA" ); })
          LABELS <- as.data.frame(LABELS)
          
          varsBySubset <- list()
